#!/bin/bash

set -ex
set -o pipefail

[ ! -d "ar" ] && mkdir "ar"
[ ! -d "var" ] && mkdir "var"

[ ! -e var/packages ] && ./update.sh
source var/packages
source var/buildscripts

if [ ! -e data/config ]
then
	set +ex
	echo "Cannot find 'data/config'. You might want to copy 'data/config.example'"
	echo "and edit it yourself."
	exit 1
fi
source data/config
export PREFIX="${PREFIX}"
source data/urls

list_packages() {
	echo "*** Known package list ***"
	for pkg in "${PACKAGES[@]}"
	do
		echo "${pkg}"
	done
}

do_unpack() {
	rm -rf "${builddir}"
	if [ ! -e "ar/${!buildar}" ]
	then
		wget -P ar -r -nd "${URL_AR_BASE}/${!buildar}"
		[ ! -e "ar/${!buildar}" ] && exit 1
	fi

	case "$(echo "${!buildar}" | sed -e "s,^.*\.\([^.]*\),\1,")" in
	gz)
		(cd var ; gunzip < "../ar/${!buildar}" | tar x) || exit 1
	;;

	bz2)
		(cd var ; bunzip2 < "../ar/${!buildar}" | tar x) || exit 1
	;;
	*)
		set +ex
		echo "Don't know how to unpack '${!buildar}'"
		exit 1
	;;
	esac
}

do_build_cname() {
	echo $1 | tr - _
}

do_build() {
	local built=0
	local buildcmd=""
	local buildvar=""
	local buildver=""
	local builddir=""
	local buildar=""
	local builddep=""

	for pkg in "${PACKAGES[@]}"
	do
		if [ "${pkg}" == "$1" ]
		then
			pkg_cname="$(do_build_cname ${pkg})"
			built=1
			buildvar="PKG_BUILD_${pkg_cname}"
			buildcmd="${!buildvar}"
			buildver="PKG_VER_${pkg_cname}"
			builddir="var/${pkg}-${!buildver}"
			buildar="PKG_AR_${pkg_cname}"
			builddep="PKG_DEPENDS_${pkg_cname}"

			[ -e "${builddir}/.built" ] && break

			for sub in ${!builddep}
			do
				do_build ${sub}
			done
			
			if [ -z "${buildcmd}" ]
			then
				echo "--- No build command for '${pkg}'"
				exit 1
			fi

			do_unpack "${!buildar}"
			export D="${builddir}"
			(
				cd "${D}"
				${buildcmd}
				[ $? -ne 0 ] && exit 1
				touch .built
			) || exit 1

			break
		fi
	done

	if [ "${built}" -eq 0 ]
	then
		set +ex
		echo "Cannot find package '$1'"
		list_packages
		exit 1
	fi
}

if [ $# -eq 0 ]
then
	set +ex
	list_packages
	exit 1
fi

for pkg in "$@"
do
	do_build ${pkg}
done

set +ex
echo "+++ Operations completed successfully."
